@@ -1,15 +1,16 @@
 """
 This file contains the API code for version 2 of the bonsai command line
 """
-__author__ = "Karthik Sankara Subramanian"
+__author__ = "Karthik Sankara Subramanian, Anil Puvvadi"
 __copyright__ = "Copyright 2019, Microsoft Corp."
 
 import click
 import json
-import sys
+import os
 import pprint
+import sys
 
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 from uuid import uuid4
 
 if sys.version_info >= (3,):
@@ -28,7 +29,8 @@ from .application_insights import (
     SkeletonApplicationInsightsHandler,
 )
 from .cookies import CookieConfiguration
-from .exceptions import UsageError
+from .exceptions import BrainServerError, UsageError
+from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 
 _LIST_BRAINS_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains"
 _CREATE_BRAIN_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}"
@@ -55,19 +57,39 @@ _DELETE_BRAIN_VERSION_URL_PATH_TEMPLATE = (
 _START_SIMULATOR_LOGGING_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/simulators/{sessionId}/startLogging"
 _STOP_SIMULATOR_LOGGING_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/simulators/{sessionId}/stopLogging"
 
-_LIST_SIM_PACKAGE_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/simulatorpackages"
-_GET_SIM_PACKAGE_URL_PATH_TEMPLATE = (
-    "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}"
+_UPLOAD_MODEL_FILE_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspacename}/packages/uploadmodelfile"
 )
 _CREATE_SIM_PACKAGE_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorpackages/{packagename}"
 )
+_LIST_SIM_PACKAGE_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/simulatorpackages"
+_GET_SIM_PACKAGE_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}"
+)
 _UPDATE_SIM_PACKAGE_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}"
 )
 _DELETE_SIM_PACKAGE_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}"
 )
+_UPLOAD_IMPORTED_MODEL_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspacename}/importedModels/uploadmodelfile"
+)
+_CREATE_IMPORTED_MODEL_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspaceid}/importedModels/{importedmodelname}"
+)
+_LIST_IMPORTED_MODEL_URL_PATH_TEMPLATE = "/v2/workspaces/{workspaceid}/importedmodels"
+
+_GET_IMPORTED_MODEL_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspaceid}/importedmodels/{importedmodelname}"
+)
+_UPDATE_IMPORTED_MODEL_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspaceid}/importedmodels/{importedmodelname}"
+)
+_DELETE_IMPORTED_MODEL_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspaceid}/importedmodels/{importedmodelname}"
+)
 
 _LIST_SIM_COLLECTION_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}/simulatorcollections"
 _GET_SIM_COLLECTION_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/simulatorpackages/{simulatorpackagename}/simulatorcollections/{collectionid}"
@@ -90,14 +112,12 @@ _DELETE_EXPORTED_BRAIN_URL_PATH_TEMPLATE = (
 _UPDATE_EXPORTED_BRAIN_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/exportedBrains/{exportedbrainname}"
 )
-
 _LIST_SIM_BASE_IMAGE_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorbaseimages"
 )
 _GET_SIM_BASE_IMAGE_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorbaseimages/{imageidentifier}"
 )
-
 _RESET_BRAIN_TRAINING_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/resetTraining"
 )
@@ -107,15 +127,12 @@ _START_BRAIN_TRAINING_URL_PATH_TEMPLATE = (
 _STOP_BRAIN_TRAINING_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/stopTraining"
 )
-
 _START_BRAIN_ASSESSMENT_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/startAssessment"
 )
 _STOP_BRAIN_ASSESSMENT_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/stopAssessment"
 )
-
-
 _LIST_SIM_SESSIONS_URL_PATH_TEMPLATE = (
     "/v2/workspaces/{workspacename}/simulatorsessions?deployment_mode=neq:Hosted"
 )
@@ -142,23 +159,22 @@ _BRAIN_VERSION_OBJECT = "BrainVersion"
 _ALL_BRAIN_VERSIONS_OBJECT = "BrainVersions"
 _SIMULATOR_PACKAGE_OBJECT = "SimulatorPackage"
 _ALL_SIMULATOR_PACKAGES_OBJECT = "SimulatorPackages"
+_IMPORTED_MODEL_OBJECT = "ImportedModel"
+_ALL_IMPORTED_MODEL_OBJECT = "ImportedModels"
 _SIMULATOR_COLLECTION_OBJECT = "SimulatorCollection"
 _INKLING_OBJECT = "Inkling"
 
+_LIST_ASSESSMENTS_URL_PATH_TEMPLATE = (
+    "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/assessments"
+)
+_CREATE_ASSESSMENT_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/assessments/{assessmentName}"
+_GET_ASSESSMENT_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/assessments/{assessmentName}"
+_DELETE_ASSESSMENT_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/assessments/{assessmentName}"
+_UPDATE_ASSESSMENT_URL_PATH_TEMPLATE = "/v2/workspaces/{workspacename}/brains/{name}/versions/{version}/assessments/{assessmentName}"
+
 log = Logger()
 
 
-class BrainServerError(Exception):
-    """
-    This is thrown for any errors.
-    """
-
-    def __init__(self, exception: Any):
-        self.exception = exception
-
-    pass
-
-
 def _handle_and_raise(response: requests.Response, e: Any, request_id: str):
     """
     This takes an exception and wraps it in a BrainServerError.
@@ -261,6 +277,7 @@ class BonsaiAPI(object):
         tenant_id: Optional[str] = None,
         api_url: Optional[str] = None,
         gateway_url: Optional[str] = None,
+        cookie_config: Optional[CookieConfiguration] = None,
     ):
         """
         Initializes the API object.
@@ -283,7 +300,10 @@ class BonsaiAPI(object):
         if gateway_url is None:
             raise ValueError("Gateway url is missing")
 
-        self.cookie_config = CookieConfiguration()
+        if cookie_config:
+            self.cookie_config = cookie_config
+        else:
+            self.cookie_config = CookieConfiguration()
         self._access_key = access_key
         self._workspace_id = workspace_id
         self.tenant_id = tenant_id
@@ -427,6 +447,14 @@ class BonsaiAPI(object):
                     allow_redirects=False,
                     timeout=self.timeout,
                 )
+            elif http_method == "POST_FILE":
+                response = self._session.post(
+                    url=url,
+                    headers=headers_out,
+                    data=data,
+                    allow_redirects=False,
+                    timeout=self.timeout,
+                )
             else:
                 raise UsageError("Unsupported HTTP Request Method")
 
@@ -452,8 +480,8 @@ class BonsaiAPI(object):
                 "timeout. Request ID: {}".format(url, self.timeout, req_id)
             )
 
+        response_dict = {}
         try:
-            response_dict = {}
             response.raise_for_status()
             self._raise_on_redirect(response)
             log.debug("{} {} results:\n{}".format(http_method, url, response.text))
@@ -475,7 +503,7 @@ class BonsaiAPI(object):
         debug: bool = False,
         output: Optional[str] = None,
         event: Optional[CustomEventInterface] = None,
-    ):
+    ) -> Any:
         """
         Wrapper for _try_http_request(), will switch to AAD authentication
         and retry if first attempt fails due to deprecated Bonsai credentials.
@@ -860,6 +888,192 @@ class BonsaiAPI(object):
 
         return self._delete(url=url, debug=debug, output=output, event=event)
 
+    def upload_model_file(self, filepath: str, debug: bool = False) -> Any:
+
+        url_path = _UPLOAD_MODEL_FILE_URL_PATH_TEMPLATE.format(
+            workspacename=self._workspace_id
+        )
+        url = urljoin(self._api_url, url_path)
+        return self.post_file(
+            url=url,
+            filename=os.path.basename(os.path.normpath(filepath)),
+            filepath=filepath,
+            debug=debug,
+        )
+
+    def upload_importedmodel(
+        self, importedmodelname: str, filepath: str, debug: bool = False
+    ) -> Any:
+
+        url_path = _UPLOAD_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspacename=self._workspace_id
+        )
+        url = urljoin(self._api_url, url_path)
+        return self.post_file(
+            url=url, filename=importedmodelname, filepath=filepath, debug=debug
+        )
+
+    # callback function to monitor the progress
+    def post_file_callback(self, monitor: MultipartEncoderMonitor):
+        log.info("NUMBER OF BYTES READY FOR UPLOAD: {}".format(monitor.bytes_read))
+
+    # uploads model file in zip format
+    def post_file(
+        self, url: str, filename: str, filepath: str, debug: bool = False
+    ) -> Any:
+
+        multipart_encoder: MultipartEncoder = MultipartEncoder(
+            fields={"file": (filename, open(filepath, "rb"))}
+        )
+        multipart_monitor: Any = MultipartEncoderMonitor(
+            multipart_encoder, self.post_file_callback
+        )
+
+        headers_out = self._get_headers()
+        headers_out.update({"Content-Type": multipart_monitor.content_type})
+        headers_out.update({"RequestId": str(uuid4())})
+        return self._http_request(
+            "POST_FILE",
+            url=url,
+            data=multipart_monitor,
+            headers=headers_out,
+            debug=debug,
+        )
+
+    def create_importedmodel(
+        self,
+        name: str,
+        uploaded_file_path: str,
+        display_name: Optional[str] = None,
+        description: Optional[str] = None,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+
+        log.information("****I am fine***")
+
+        url_path = _CREATE_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspaceid=workspace if workspace else self._workspace_id,
+            importedmodelname=name,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_CREATE_ACTION, _IMPORTED_MODEL_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_IMPORTED_MODEL_OBJECT],
+        )
+
+        data = {
+            "displayName": display_name,
+            "description": description,
+            "uploadedFilePath": uploaded_file_path,
+        }
+        return self._put(url=url, data=data, debug=debug, output=output, event=event)
+
+    def list_importedmodels(
+        self,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Getting list of imported models for {}...".format(self._workspace_id)
+        )
+        url_path = _LIST_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspaceid=workspace if workspace else self._workspace_id
+        )
+        url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_VIEW_ACTION, _ALL_IMPORTED_MODEL_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ALL_IMPORTED_MODEL_OBJECT],
+        )
+
+        return self._get(url=url, debug=debug, output=output, event=event)
+
+    def get_importedmodel(
+        self,
+        name: str,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Getting details about imported model {} in workspace {}...".format(
+                name, self._workspace_id
+            )
+        )
+
+        url_path = _GET_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspaceid=workspace if workspace else self._workspace_id,
+            importedmodelname=name,
+        )
+        url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_VIEW_ACTION, _IMPORTED_MODEL_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_IMPORTED_MODEL_OBJECT],
+        )
+
+        return self._get(url=url, debug=debug, output=output, event=event)
+
+    def update_importedmodel(
+        self,
+        name: str,
+        display_name: Optional[str] = None,
+        description: Optional[str] = None,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Updating details for imported models {} in workspace {}...".format(
+                name, self._workspace_id
+            )
+        )
+        url_path = _UPDATE_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspaceid=workspace if workspace else self._workspace_id,
+            importedmodelname=name,
+        )
+        url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_UPDATE_ACTION, _IMPORTED_MODEL_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_IMPORTED_MODEL_OBJECT],
+        )
+
+        data = {
+            "displayName": display_name,
+            "description": description,
+        }
+        response = self._patch(
+            url=url, data=data, debug=debug, output=output, event=event
+        )
+        return response
+
+    def delete_importedmodel(
+        self,
+        name: str,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug("Deleting imported models {}".format(name))
+        url_path = _DELETE_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
+            workspaceid=workspace if workspace else self._workspace_id,
+            importedmodelname=name,
+        )
+        url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_DELETE_ACTION, _IMPORTED_MODEL_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_IMPORTED_MODEL_OBJECT],
+        )
+        response = self._delete(url=url, debug=debug, output=output, event=event)
+        return response
+
     def create_sim_package(
         self,
         name: str,
@@ -1038,6 +1252,8 @@ class BonsaiAPI(object):
         max_instance_count: Optional[str] = None,
         auto_scaling: Optional[str] = None,
         auto_termination: Optional[str] = None,
+        log_session_count: Optional[str] = None,
+        include_system_logs: bool = False,
         workspace: Optional[str] = None,
         debug: bool = False,
         output: Optional[str] = None,
@@ -1064,6 +1280,12 @@ class BonsaiAPI(object):
             )
         )
 
+        simulatorLogConfig = json.loads(
+            '{{"sessionCount": "{}", "includeSystemLogs": "{}" }}'.format(
+                log_session_count, include_system_logs
+            )
+        )
+
         data = {
             "purpose": purpose,
             "description": description,
@@ -1076,6 +1298,7 @@ class BonsaiAPI(object):
             "maxInstanceCount": max_instance_count,
             "autoScaling": auto_scaling,
             "autoTermination": auto_termination,
+            "simulatorLogConfig": simulatorLogConfig,
         }
 
         return self._post(url=url, data=data, debug=debug, output=output, event=event)
@@ -1223,6 +1446,7 @@ class BonsaiAPI(object):
         name: str,
         version: int = 1,
         workspace: Optional[str] = None,
+        concept_names: Optional[List[str]] = None,
         debug: bool = False,
         output: Optional[str] = None,
     ):
@@ -1239,7 +1463,11 @@ class BonsaiAPI(object):
             ObjectType=[_BRAIN_VERSION_OBJECT],
         )
 
-        return self._post(url=url, debug=debug, output=output, event=event)
+        data: Optional[Dict[str, Any]] = (
+            {"concepts": concept_names} if concept_names else None
+        )
+
+        return self._post(url=url, debug=debug, output=output, event=event, data=data)
 
     def stop_training(
         self,
@@ -1268,7 +1496,8 @@ class BonsaiAPI(object):
         self,
         name: str,
         session_id: str,
-        session_count: int,
+        log_session_count: int,
+        include_system_logs: bool,
         version: int = 1,
         workspace: Optional[str] = None,
         debug: bool = False,
@@ -1281,7 +1510,10 @@ class BonsaiAPI(object):
             sessionId=session_id,
         )
 
-        data = {"sessionCount": session_count}
+        data = {
+            "sessionCount": log_session_count,
+            "includeSystemLogs": include_system_logs,
+        }
 
         url = urljoin(self._api_url, url_path)
 
@@ -1393,6 +1625,7 @@ class BonsaiAPI(object):
         workspace: Optional[str] = None,
         debug: bool = False,
         output: Optional[str] = None,
+        export_type: Optional[str] = None,
     ):
         log.debug("Creating a new exported brain {}".format(name))
         url_path = _CREATE_EXPORTED_BRAIN_URL_PATH_TEMPLATE.format(
@@ -1404,6 +1637,7 @@ class BonsaiAPI(object):
             ObjectUri=[url_path],
             ObjectType=[_EXPORTED_BRAIN_OBJECT, _BRAIN_VERSION_OBJECT],
         )
+        export_type = export_type or "Predictor"
 
         data = {
             "name": name,
@@ -1415,6 +1649,7 @@ class BonsaiAPI(object):
             "brainVersion": brain_version,
             "displayName": display_name,
             "description": description,
+            "exportType": export_type,
         }
 
         return self._post(url=url, data=data, debug=debug, output=output, event=event)
@@ -1590,6 +1825,166 @@ class BonsaiAPI(object):
 
         return self._patch(url=url, data=data, debug=debug, output=output)
 
+    def start_assessmentv2(
+        self,
+        name: str,
+        brain_name: str,
+        version: int,
+        concept_name: str,
+        scenarios: str,
+        episode_iteration_limit: int,
+        maximum_duration_in_minutes: int,
+        display_name: Optional[str] = None,
+        description: Optional[str] = None,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug("Creating a new assessment {}".format(name))
+        url_path = _CREATE_ASSESSMENT_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+            assessmentName=name,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        data = {
+            "name": name,
+            "concept": concept_name,
+            "scenarios": scenarios,
+            "displayName": display_name,
+            "description": description,
+            "episodeIterationLimit": episode_iteration_limit,
+            "maximumDurationInMinutes": maximum_duration_in_minutes,
+        }
+
+        return self._post(url=url, data=data, debug=debug, output=output)
+
+    def list_assessment(
+        self,
+        brain_name: str,
+        version: int,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Getting list of assessments for brain {} version {} in workspace...".format(
+                brain_name, version, self._workspace_id
+            )
+        )
+        url_path = _LIST_ASSESSMENTS_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        return self._get(url=url, debug=debug, output=output)
+
+    def get_assessment(
+        self,
+        name: str,
+        brain_name: str,
+        version: int,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Getting details about assessment {} for brain {} version {} in workspace {}...".format(
+                name, brain_name, version, self._workspace_id
+            )
+        )
+
+        url_path = _GET_ASSESSMENT_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+            assessmentName=name,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        return self._get(url=url, debug=debug, output=output)
+
+    def update_assessment(
+        self,
+        name: str,
+        brain_name: str,
+        version: int,
+        display_name: Optional[str] = None,
+        description: Optional[str] = None,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Updating details about assessment {} for brain {} version {} in workspace {}...".format(
+                name, brain_name, version, self._workspace_id
+            )
+        )
+        url_path = _UPDATE_ASSESSMENT_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+            assessmentName=name,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        data = {"displayName": display_name, "description": description}
+        return self._put(url=url, data=data, debug=debug, output=output)
+
+    def stop_assessment_v2(
+        self,
+        name: str,
+        brain_name: str,
+        version: int,
+        state: Optional[str] = None,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Updating details about assessment {} for brain {} version {} in workspace {}...".format(
+                name, brain_name, version, self._workspace_id
+            )
+        )
+        url_path = _UPDATE_ASSESSMENT_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+            assessmentName=name,
+        )
+        url = urljoin(self._api_url, url_path)
+
+        data = {"state": state}
+        return self._put(url=url, data=data, debug=debug, output=output)
+
+    def delete_assessment(
+        self,
+        name: str,
+        brain_name: str,
+        version: int,
+        workspace: Optional[str] = None,
+        debug: bool = False,
+        output: Optional[str] = None,
+    ):
+        log.debug(
+            "Deleting assessment {} for brain {} version {} in workspace {}...".format(
+                name, brain_name, version, self._workspace_id
+            )
+        )
+
+        url_path = _DELETE_ASSESSMENT_URL_PATH_TEMPLATE.format(
+            workspacename=workspace if workspace else self._workspace_id,
+            name=brain_name,
+            version=version,
+            assessmentName=name,
+        )
+        url = urljoin(self._api_url, url_path)
+        return self._delete(url=url, debug=debug, output=output)
+
     def _raise_on_redirect(self, response: requests.Response):
         """Raises an HTTPError if the response is 301.
 
